import sys
import os


ERRORS_DEFS = {
	"GENERIC" : "Generic error raised!",

	"BOT_NOT_FOUND" : "Bot not instantiated!",
	
	"CONNECTION_ERROR" : "Cannot establish connection with the bot",
	
	"MAX_TRIES_REACHED" : "Cannot connect. Max number of tries limit reached.",
	"API_KEY_MISSING" : "No Api key to be found!",

	"AUTH_ERROR" : "Authentication error!",
	"AUTH_FAIL_ID" : "Authentication Failure: ID not valid!",
	"AUTH_FAIL_USER" : "Authentication Failure: Username not valid!",
	"AUTH_FAIL_NAME" : "Authentication Failure: First name not valid!",
	
	"ROLL_STRING_ERROR" : "Syntax error on roll string!",
	
	"ROLL_TOO_MUCH" : "Too many dice rolled",
}

class GenericError(Exception):

	def __init__(self, code="GENERIC", data=None):
		self.code = code
		
		self.message = ERRORS_DEFS[self.code]

		self.data = data

	def get_message(self):
		return "{0}: {1}".format(self.code, self.message)
		
	def dump(self):
		s = self.get_message()

		sys.stderr.flush()
		sys.stderr.write(s + os.linesep)
		sys.stderr.flush()

		return s

	def __str__(self):
		return self.get_message()
	
	def __repr__(self):
		return "<GenericError {}>".format(self.code)

###########################

class BotNotFound(GenericError):

	def __init__(self, data=None, code="BOT_NOT_FOUND"):
		super(BotNotFound, self).__init__(code=code, data=data)

	def __repr__(self):
		return "<BotNotFound {}>".format(self.code)


#########################################


class ConnectionError(GenericError):

	def __init__(self, data=None, code="CONNECTION_ERROR"):
		super(ConnectionError, self).__init__(code=code, data=data)

	def __repr__(self):
		return "<ConnectionError {}>".format(self.code)



class ConnectionMaxTries(ConnectionError):

	def __init__(self, data=None, code="MAX_TRIES_REACHED"):
		super(ConnectionMaxTries, self).__init__(code=code, data=data)

	def __repr__(self):
		return "<ConnectionMaxTries {}>".format(self.code)

		
		
class ConnectionApiKeyMissing(ConnectionError):

	def __init__(self, data=None, code="API_KEY_MISSING"):
		super(ConnectionApiKeyMissing, self).__init__(code=code, data=data)

	def __repr__(self):
		return "<ConnectionApiKeyMissing {}>".format(self.code)

#################################

class ConnectionAuthentication(GenericError):
	def __init__(self, data=None, code="AUTH_ERROR"):
		super(ConnectionAuthentication, self).__init__(code=code, data=data)

	def __repr__(self):
		return "<ConnectionAuthentication {}>".format(self.code)


class ConnectionAuthenticationID(ConnectionAuthentication):

	def __init__(self, data=None, code="AUTH_FAIL_ID"):
		super(ConnectionAuthenticationID, self).__init__(code=code, data=data)


	def __repr__(self):
		return "<ConnectionAuthenticationID {}>".format(self.code)

		
class ConnectionAuthenticationUser(ConnectionAuthentication):

	def __init__(self, data=None, code="AUTH_FAIL_USER"):
		super(ConnectionAuthenticationUser, self).__init__(code=code, data=data)


	def __repr__(self):
		return "<ConnectionAuthenticationUser {}>".format(self.code)

class ConnectionAuthenticationName(ConnectionAuthentication):

	def __init__(self, data=None, code="AUTH_FAIL_NAME"):
		super(ConnectionAuthenticationName, self).__init__(code=code, data=data)

	def __repr__(self):
		return "<ConnectionAuthenticationName {}>".format(self.code)


#############################


class RollStringError(GenericError):

	def __init__(self, data=None, code="ROLL_STRING_ERROR"):
		super(RollStringError, self).__init__(code=code, data=data)

	def __repr__(self):
		return "<RollStringError {}>".format(self.code)



class RollTooManyDice(GenericError):

	def __init__(self, data=None, code="ROLL_TOO_MUCH"):
		super(RollTooManyDice, self).__init__(code=code, data=data)

	def __repr__(self):
		return "<RollTooManyDice {}>".format(self.code)

