# - coding: utf-8 -

import sys
import time

import telepot

from settings import *
from utils import *
from errors import ConnectionError, ConnectionMaxTries, ConnectionApiKeyMissing

import handler
###############################################


class BotManager(object):
	
	api_key = ""
	
	bot = None
	authenticated = False
	
	last_id = 0
	
	def __init__(self, api_key, max_tries=3, auto_start=1):
		self.bot = None
		self.api_key = api_key
		self.authenticated = False
		
		if auto_start:
			self.init_bot(tries=max_tries)
			
	
	def init_bot(self, key="", tries=3):
		key = key or self.api_key

		if not key:
			raise ConnectionApiKeyMissing()

		self.bot = BotManager.create_bot(key, tries)

		if self.bot:
			self.bot.manager = self
		

		return self.bot


	def check_authentication(self, id_bot, username, first_name):

		self.check_bot_instance()
		
		try:
			d = self.bot.getMe()
		except:
			raise ConnectionError()

		if d["id"] != id_bot:
			raise ConnectionAuthenticationID()
		
		if d["username"] != username:
			raise ConnectionAuthenticationUser()
	
		if d["first_name"] != first_name:
			raise ConnectionAuthenticationName()


		self.authenticated = True

		return self.bot


	def check_bot_instance(self):
		if not self.bot:
			raise BotNotFound()

	
	
	def msg_handler_defer(self, msg):
		self.msg_handler(msg, self.bot)
	
	
	def init_handler(self, handler):
		self.check_bot_instance()

		self.msg_handler = handler
		
		self.bot.message_loop(self.msg_handler_defer)

		return self.bot


	@classmethod
	def create_bot(self, api_key, tries=3):
		bot = None

		for x in xrange(tries):
			print("Try {} of {}: ".format(x+1, tries)),

			try:
				bot = telepot.Bot(api_key)
				bot.getMe()
				print("OK!")
				break
		
			except:	
				print("Fail!")
				time.sleep(3)

		else:
			raise ConnectionMaxTries({"key": api_key, "tries":tries})
		
		return bot


	def send_msg(self, msg, chat=-35789852, nosound=True):
		self.bot.sendMessage(chat, msg, "HTML", True, bool(nosound))
		
###############################################



def get_basic_bot():
	BOT = BotManager(API_KEY, START_MAX_TRIES)
	BOT.check_authentication(BOT_ID, BOT_USERNAME, BOT_FIRST_NAME)

	return BOT

if __name__ == "__main__":
	#BOT = BotManager(API_KEY, START_MAX_TRIES)
	#BOT.check_authentication(BOT_ID, BOT_USERNAME, BOT_FIRST_NAME)

	BOT = get_basic_bot()
	BOT.last_id = LAST_MSG_ID
	
	BOT.init_handler(handler.main_handler) #from handler module
	
	
	try:
		while 1:
			time.sleep(10)
	except:
		
		f = open("last_id.txt", "w")
		f.write(str(BOT.last_id))
		f.close()	
	
		exit_msg = " EXIT! LAST MESSAGE ID => {} ".format(BOT.last_id)
		print("*" * 70)
		print(">" * 10 + exit_msg + "<" * 10)
		print("*" * 70)




#BOT = BotManager(API_KEY, START_MAX_TRIES)
#BOT.check_authentication(BOT_ID, BOT_USERNAME, BOT_FIRST_NAME)
#BOT.bot.sendMessage(-35789852,"Ma insomma, terrI !!!", "HTML", True, True)


