
__email__ = "Kadmillos@gmail.com"
__author__ = "Kadmillos"
__version__ = "0.9"

import sys
import random
import re


from errors import RollStringError, RollTooManyDice
########################################################


########################################################

ROLL_SPLITTER = re.compile(r"([+-])+?").split

ROLL_CHECK = re.compile(r"^(\d+)?(?:\/(\d+))?[dD](\d+)(\!\d*)?(?:([-+])(\d+))?$").match
ROLL_POOL = re.compile(r"^(?:(\d+)[dD](\d+)(\!\d*)?)(?:\(?(\-?\d+)?(?:\,(\d+)?)?(?:\,(\d+))?\))?$").match

MAX_ROLL_DICE = 61

def roll_die(vmin, vmax, opts=None):
	random.seed()
	return random.randint(vmin, vmax)

##################################################

def bang_roll(vmin, vmax, vbang, opts=None):
	rv = roll_die(vmin, vmax, opts)
	
	if vbang < 2:
		return rv

	value = rv
	while rv >= vbang:
		rv = roll_die(vmin, vmax, opts)
		value += rv
	
	return value



###############
def roll_margin(vmin, vmax, opts=None):
	if not opts:
		return roll_die(vmin, vmax, opts)

	
	if "bang" in opts:
		vbang = opts["bang"] or vmax
		return bang_roll(vmin, vmax, vbang, opts)

			
	return value


############################################################


def parse_options_string(opts):
	opts_d = {}

	if "!" in opts:
		vbang = opts[1:]
		opts_d["bang"] = int(vbang) if vbang else None

	return opts_d


def parse_roll_string(dstring):

	if dstring.isdigit():
		return (0, 0, 0, None, "FIXED", int(dstring))

	res = ROLL_CHECK(dstring.strip())
	if not res:
		return None

	(qta, tot, die, opts, mod, val) = res.groups()

	try:
		qta = int(qta)
	except:
		qta = 1
	
	try:
		tot = int(tot)	
	except:
		tot = qta
		
	tot = max(tot, qta)

	if val:
		val = int(val)
	else:
		val = 0

	if die.isdigit():
		die = int(die)

	if opts:
		opts = parse_options_string(opts)

	return (qta, tot, die, opts, mod, val)


def parse_pool_string(dstring):
	
	res = ROLL_POOL(dstring.strip())
	if not res:
		return None
	
	(qta, die, opts, threshold, take_high, take_low) = res.groups()

	try:
		qta = int(qta)
	except:
		qta = 1

	if die:
		die = int(die)
		
	try:
		threshold = int(threshold)
	except:
		threshold = 1

	try:
		take_high = int(take_high)
	except:
		take_high = 0
		
	try:
		take_low = int(take_low)
	except:
		take_low = 0
		
	if opts:
		opts = parse_options_string(opts)

	return (qta, die, opts, threshold, take_high, take_low)


def pool(dstring, total_dice=0):
	
	res = parse_pool_string(dstring)
	if not res:
		raise RollStringError()
	
	(qta, die, opts, threshold, high, low) = res

	if (total_dice + qta) > MAX_ROLL_DICE:
		raise RollTooManyDice()

	reverse = (threshold < 0)
	if reverse:
		threshold = (0 - threshold)
		
	threshold_active = (threshold > 1)
	
	rolls = []
	valid_rolls = []
	high_rolls = []
	low_rolls = []
	
	for x in xrange(qta):
		random.seed()

		value = roll_margin(1, die, opts)

		if high and value >= high:
			high_rolls.append(value)
			continue

		if low and value <= low:
			low_rolls.append(value)
			continue
		
		if reverse:
			if value <= threshold:
				valid_rolls.append(value)
				continue
		else:
			if threshold_active and value >= threshold:
				valid_rolls.append(value)
				continue

		rolls.append(value)

	rolls.sort(reverse=True)

	if threshold_active:
		valid_rolls.sort(reverse=True)
		if reverse:
			rolls.append(valid_rolls)
		else:
			rolls.insert(0, valid_rolls)
	
	if high:
		high_rolls.sort(reverse=True)
		rolls.insert(0, high_rolls)

	if low:
		low_rolls.sort(reverse=True)
		rolls.append(low_rolls)
		
	
	if reverse:
		total = len(valid_rolls) + len(low_rolls)
	else:
		total = len(valid_rolls) + len(high_rolls)
		
	return (total, rolls, threshold_active, high, low) #number of success, total_rolls, threshold calculation


def roll(dstring, total_dice=0):

	res = parse_roll_string(dstring)
	if not res:
		raise RollStringError()
	
	(qta, tot, die, opts, mod, val) = res
	
	
	if (total_dice + tot) > MAX_ROLL_DICE:
		raise RollTooManyDice()
	
	
	if mod == "FIXED":
		return (val, [val], 0, 0)
	
	if mod == "-":
		result = (0 - val)
	
	else:		
		result = val

	rolls = [roll_margin(1, die, opts) for x in xrange(int(max(tot, qta)))]
	
	rolls.sort(reverse=True)
	rolls[:qta] = [rolls[:qta]]
	
	result += sum(rolls[0])

	if len(rolls) == 1:
		rolls = rolls[0]

	return (result, rolls, val, tot)
	



def analyze_roll(dstring):
	
	dstring_parts = [p.strip() for p in ROLL_SPLITTER(dstring)]

	return dstring_parts


def flatten_roll_results(rolls):
	
	return rolls[0] + rolls[1:]


def multi_roll(roll_string, cheat_lvl=None):
		
	total_result = 0
	total_rolls = []

	result_add = True


	total_dice = 0	

	for str_part in analyze_roll(roll_string):

		if not str_part:
			continue
		
		if str_part == "+":
			result_add = True
			continue
		
		if str_part == "-":
			result_add = False
			continue
		
		(result, rolls, val, dice_used) = roll(str_part, total_dice)
		
		total_dice += dice_used
		
		if not result_add:
			result = (0 - result)

		total_rolls.append((result, rolls, val, str_part))

		total_result += result
			
	return (total_result, total_rolls)



def pool_roll(dstring, cheat_lvl=None):

	total_roll = 0
	
	return pool(dstring, total_roll)


def print_rolls(dstring, cols, rows, separator="\t", lineterminator="\r\n"):

	c = []
	write = c.append
	head = "Roll %(dstring)s:" % vars()
	write(head + lineterminator)
	write("-" * (int(len(head) * 1.5) + 1) + lineterminator)

	for r in xrange(rows):
		for x in xrange(cols):
			write(str(roll(dstring)[0]) + separator)
		write(lineterminator)
	write(lineterminator)
	
	return "".join(c)

if __name__ == "__main__":
	try:
		dstring = sys.argv[1]
	except:
		print "ERROR: indicare il tipo di roll da effettuare (possibilmente tra virgolette)"
		print 'Es: roller.py "3/4d6+2" oppure roller.py "5-20"'
		sys.exit(1)

	(result, rolls, val) = roll(dstring)
	print
	print "Roll:", dstring,
	print  "->", rolls,
	print ("%+d" % (val)) if val else "",
	print  "=>", result  
	print
